package com.JfreeChart;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.nio.file.Files;
import java.nio.file.Path;
import java.text.NumberFormat;

import javax.swing.JButton;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;


import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.time.Millisecond;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RefineryUtilities;


public class DynamicTimeSeriesChart extends ApplicationFrame implements ActionListener{

	private JFormattedTextField timeInput;
	private JFormattedTextField timeValueField;
	private JFormattedTextField error_message;
	private JFormattedTextField timeValueType;
	private JFormattedTextField fsrValue;
	private JFormattedTextField fsrAverage;
	
	
	final JPanel timeInputPanel;
	final JPanel locationSelections;
	final JPanel textPanel;
	final JPanel compareDataPanel;
	final JPanel selectionPanel;
	final JPanel timescaleSelections;
	final JPanel fsrPanel;
	
	final ButtonGroup timescale;
	final ButtonGroup location;
	final JButton startButton;
	
	//private JLabel amountLabel;
	//private static String amountString = "Duration: ";
	
	public final JFreeChart chart;
	public final JFrame display;
	
	
	
	private NumberFormat timeInputFormat;
	private String fileNamePNG;
	private Path filenameBackup;
	
	public boolean startLogging = false;
	public boolean compare = false;
	public int workstation = 0;
	public boolean realTime = false;
	public int timeSpecifier = 0;
	public int timeValue = 0;
    int[] selections;

	
	boolean enablePlot = false;
	
	String compareStr = "compare data chosen";
	String notCompareStr = "no comparison";
	
	String ntnuPC = "Workstation: NTNU";
	String laptopPC = "Workstation: Laptop";
	
	String hourStr = "hours";
	String minStr = "minutes";
	String secStr = "seconds";
	String indef = "indefinitely";
	
	String start = "start";
	String close = "close";
	
	
	
	
	
	public void setFilenamePNG(String f){
		fileNamePNG = f;
	}
	
	public void getFilenameBackupPATH(Path f) {
		filenameBackup = f;
		Path path = filenameBackup;
		if(Files.notExists(path)){
			System.out.println("");
				
			
			
			JOptionPane.showMessageDialog(display,
				    "Path does not exist.\n"
					+ "Please choose correct workstation.",
				    "Backup Error",
				    JOptionPane.ERROR_MESSAGE);
			Component[] components2 = locationSelections.getComponents();
			  for (Component component : components2) {
	                component.setEnabled(true);
	               
	            }
	            location.clearSelection();
	            startLogging = false;
			
		}
		
	}
	
	//unused
	public void enableAll(){
		Component[] components = timescaleSelections.getComponents();
    	Component[] components2 = locationSelections.getComponents();
    	Component[] components3 = compareDataPanel.getComponents();
    	Component[] components4 = timeInputPanel.getComponents();
    	
            for (Component component : components) {
                component.setEnabled(true);
               
            }
            
            
            for (Component component : components2) {
                component.setEnabled(true);
               
            }
            for (Component component : components3) {
                component.setEnabled(true);
                
            }
            for (Component component : components4) {
                component.setEnabled(true);
                
            }
            
	}
	
	
    private TimeSeries series;
    
    public float lastValue = 0;
    
    
    public float maxValue = 100;
    
    
    public DynamicTimeSeriesChart(final String title) {

        super(title);
        this.series = new TimeSeries("FSR value", Millisecond.class);
        final TimeSeriesCollection dataset = new TimeSeriesCollection(this.series);
        chart= createChart(dataset);

        final ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setEnabled(false);
        
        startButton = new JButton("Start session");
        startButton.setEnabled(false);
        startButton.setActionCommand("START_LOG");
        startButton.addActionListener(this);
        
        
        final JButton exitButton = new JButton("Close");
        exitButton.setActionCommand("Close_Demo");
        exitButton.addActionListener(this);

        
        
        final JRadioButton compare = new JRadioButton("Do");
        compare.addActionListener(this);
        compare.setActionCommand("COMP");
        final JRadioButton unCompare = new JRadioButton("Don't");
        unCompare.setActionCommand("UCOMP");
        unCompare.addActionListener(this);
        
        
        final ButtonGroup compareData = new ButtonGroup();
        compareData.add(compare);
        compareData.add(unCompare);
       
        
		final JRadioButton ntnu = new JRadioButton("NTNU");
		ntnu.setActionCommand("WORKSTATION");
		ntnu.addActionListener(this);
		
		final JRadioButton laptop = new JRadioButton("Laptop");
		laptop.addActionListener(this);
		laptop.setActionCommand("LAPTOP");
		
		
		
		location = new ButtonGroup();
		location.add(ntnu);
		location.add(laptop);
		
		
		
		
		
		final JRadioButton hours = new JRadioButton("Hours");
		hours.setActionCommand("HOURS");
		hours.addActionListener(this);

		final JRadioButton minutes = new JRadioButton("Minutes");
		minutes.setActionCommand("MINUTES");
		minutes.addActionListener(this);

		final JRadioButton seconds = new JRadioButton("Seconds");
		seconds.setActionCommand("SECONDS");
		seconds.addActionListener(this);
		
		final JRadioButton indefinitely = new JRadioButton("Indefinitely");
		indefinitely.setActionCommand("INDEF_TIME");
		indefinitely.addActionListener(this);
		
		
		
		
		timescale = new ButtonGroup();
		timescale.add(hours);
		timescale.add(minutes);
		timescale.add(seconds);
		timescale.add(indefinitely);

		
		
		error_message = new JFormattedTextField();
		error_message.setPreferredSize(new Dimension(1,21));
		error_message.setBackground(getBackground());
		error_message.setBorder(null);
		error_message.setEditable(false);
		
		
		
		//Enabling user input
		timeInput = new JFormattedTextField ();
		timeInput.setDisabledTextColor(getBackground());
		timeInput.setPreferredSize(new Dimension(1,21));
		timeInput.setActionCommand("TEXT");
		timeInput.addActionListener(this);
		timeInput.setEnabled(false);
		
		timeInput.setMaximumSize(new Dimension(1,1));
		timeInput.setHorizontalAlignment(SwingConstants.RIGHT);
		JLabel timeInputLabel = new JLabel("Duration: ");
		timeInputLabel.setLabelFor(timeInput);
        
		
		
		//Visualizing the specified duration
		timeValueField = new JFormattedTextField ();
		timeValueField.setBackground(getBackground());
		timeValueField.setBorder(null);
		timeValueField.setEditable(false);
		timeValueField.setHorizontalAlignment(SwingConstants.RIGHT);
		
		
		//Hours, minutes or seconds indicator
		timeValueType = new JFormattedTextField ();
		timeValueType.setBackground(getBackground());
		timeValueType.setBorder(null);
		timeValueType.setEditable(false);
	
		
		//Visualizing the FSR value
		fsrValue = new JFormattedTextField ();
		fsrValue.setBackground(getBackground());
		fsrValue.setBorder(null);
		fsrValue.setEditable(false);
		fsrValue.setHorizontalAlignment(SwingConstants.RIGHT);
		JLabel fsrValueLabel = new JLabel("Current value: ");
		fsrValueLabel.setLabelFor(fsrValue);
		
		
		
		//Visualizing the FSR Average
		fsrAverage = new JFormattedTextField ();
		fsrAverage.setBackground(getBackground());
		fsrAverage.setBorder(null);
		fsrAverage.setEditable(false);
		fsrAverage.setHorizontalAlignment(SwingConstants.RIGHT);
		JLabel fsrAverageLabel = new JLabel("Average: ");
		fsrAverageLabel.setLabelFor(fsrAverage);
				
		
		fsrPanel = new JPanel(new GridLayout(2,2));
		fsrPanel.add(fsrAverageLabel);
		fsrPanel.add(fsrAverage);
		fsrPanel.add(fsrValueLabel);
		fsrPanel.add(fsrValue);
		
		
		timeInputPanel = new JPanel(new GridLayout(1,4));
		timeInputPanel.add(timeInputLabel);
		timeInputPanel.add(timeInput);
		timeInputPanel.add(timeValueField);
		timeInputPanel.add(timeValueType);
		
		
		//Combining text-input panels and error message
		textPanel = new JPanel(new GridLayout(0,1));
		textPanel.add(timeInputPanel);
		textPanel.add(error_message);
		textPanel.add(fsrPanel);
		textPanel.add(startButton);
        
		compareDataPanel = new JPanel(new GridLayout(0,1));
		compareDataPanel.setBorder(BorderFactory.createTitledBorder("Compare data"));
		compareDataPanel.add(compare);
		compareDataPanel.add(unCompare);
		
        locationSelections = new JPanel(new GridLayout(0,1));
        locationSelections.setBorder(BorderFactory.createTitledBorder("Workstation"));
		locationSelections.add(ntnu);
		locationSelections.add(laptop);
		
       
		
		timescaleSelections = new JPanel(new GridLayout(0,1));
        timescaleSelections.setBorder(BorderFactory.createTitledBorder("Timescale"));
        timescaleSelections.add(hours);
        timescaleSelections.add(minutes);
        timescaleSelections.add(seconds);
        timescaleSelections.add(indefinitely);

       
        selectionPanel = new JPanel(new GridLayout(0,1));
        selectionPanel.add(locationSelections);
        selectionPanel.add(compareDataPanel);
        selectionPanel.add(timescaleSelections);
        selectionPanel.add(textPanel);

        display = new JFrame();
        add(chartPanel,BorderLayout.CENTER);
        add(selectionPanel, BorderLayout.LINE_START);
        add(exitButton,BorderLayout.PAGE_END);
        chartPanel.setPreferredSize(new Dimension(750, 250));
        
        
        

        
        selections = new int[4]; // generate an empty array where every position needs to have a 1 to be able to start
    	
        

        pack();
        RefineryUtilities.positionFrameOnScreen(this, LEFT_ALIGNMENT, TOP_ALIGNMENT);
        setVisible(true);
        
        
        
        
    }
    
    
  

    
    private JFreeChart createChart(final XYDataset dataset) {
        final JFreeChart result = ChartFactory.createTimeSeriesChart(
            "Real time plot FSR", 	//title
            "Time", 				//x-axis
            "Voltage",				//y-axis
            dataset, 				//dataset
            true, 					//legend
            true, 					//tool-tips
            false					//urls
        );
       
        final XYPlot plot = result.getXYPlot();
        ValueAxis axis = plot.getDomainAxis();
        axis.setAutoRange(true);
          axis = plot.getRangeAxis();
        axis.setAutoRange(true);
        return result;
    }
    
    
 
 public void addAverageValue(String data){	
    	fsrAverage.setText(data);
    	
    	
    }
    
    public void addValue(float data){	
    	this.lastValue = data;
    	fsrValue.setText(""+lastValue);
    	
    	
    	this.series.addOrUpdate(new Millisecond(), this.lastValue);
    	
    }
    

    
    
    public void actionPerformed(final ActionEvent e) { 	
    			

    	
        if (e.getActionCommand().equals("Close_Demo")) {
        	
       	
        	
        	System.out.println(close);
        	System.exit(0);
        	
        }
        
        if(e.getActionCommand().equals("START_LOG")){
        	startLogging = true;
        	enablePlot = true;
        	Component[] components = timescaleSelections.getComponents();
        	Component[] components2 = locationSelections.getComponents();
        	Component[] components3 = compareDataPanel.getComponents();
        	Component[] components4 = timeInputPanel.getComponents();
        	
	            for (Component component : components) {
	                component.setEnabled(false);
	            }
	            for (Component component : components2) {
	                component.setEnabled(false);
	            }
	            for (Component component : components3) {
	                component.setEnabled(false);
	            }
	            for (Component component : components4) {
	                component.setEnabled(false);
	            }
        	System.out.println(start);
        
        	
        }
        
        
        
        
        //action listener for location
        if(e.getActionCommand().equals("WORKSTATION")){
        	workstation = 1;
        	selections[0] = 1;
        	System.out.println(ntnuPC);
        }else if(e.getActionCommand().equals("LAPTOP")){
        	workstation = 2;
        	selections[0] = 1;
        	System.out.println(laptopPC);
        }
        
        //action listener for timescale
        if(e.getActionCommand().equals("HOURS")){
        	timeSpecifier = 1;
        	timeValueType.setText(" hours");
        	System.out.println(hourStr);
        	timeInput.setEnabled(true);
        	selections[1] = 1;
        }else if(e.getActionCommand().equals("MINUTES")){
        	timeSpecifier = 2;
        	timeValueType.setText(" minutes");
        	System.out.println(minStr);
        	timeInput.setEnabled(true);
        	selections[1] = 1;
        }else if(e.getActionCommand().equals("SECONDS")){
        	timeSpecifier = 3;
        	timeValueType.setText(" seconds");
        	System.out.println(secStr);
        	timeInput.setEnabled(true);
        	selections[1] = 1;
        }else if(e.getActionCommand().equals("INDEF_TIME")){
        	timeInput.setEnabled(false);
        	timeInput.setBackground(getBackground());
        	timeSpecifier = 4;
        	timeValue = 0;
        	timeValueType.setText("");
        	timeValueField.setText("");
        	System.out.println(indef);
        	selections[1] = 1;
        	selections[2] = 1;
        }
        
        
        
        //action listener for text input
        if(e.getActionCommand().equals("TEXT")){
        	error_message.setText(null);
        	timeInputFormat = NumberFormat.getNumberInstance();
        	selections[2] = 1;
        	String strTimeInput = timeInput.getText();
        	
    		try {
				timeValue = Integer.parseInt(strTimeInput);
				
				if(timeValue<0){
					error_message.setText("     #positive integers only#");
					timeValue = 0;
				}else{
					timeValueField.setText(""+timeValue);
				}
				
			} catch (NumberFormatException e1) {
				error_message.setText("     #positive integers only#");
				e1.printStackTrace();
			}
    		
        	timeInput.setText(null);
        	System.out.println(timeValue);
        	
        }
        
        if(e.getActionCommand().equals("COMP")){
        	selections[3] = 1;
        	compare = true;
        	System.out.println("Compare: " + compare);
        }else if(e.getActionCommand().equals("UCOMP")){
        	selections[3] = 1;
        	compare = false;
        	System.out.println("Compare: " + compare);
        }
        
       
        boolean check = true;
        int len = selections.length;
        for(int a = 0; check && a < len; a++) {
            check = selections[a]==1 && check;
        }
        
        if (check == true){
        	startButton.setEnabled(true);
        	if(e.getActionCommand().equals("START_LOG")){
        		startButton.setEnabled(false);
        	}
        }
    }


}








           